<?php

namespace Wappointment\Controllers;

use Wappointment\Services\Client;
use Wappointment\Validators\HttpRequest\Booking;
use Wappointment\Validators\HttpRequest\BookingAdmin;
use Wappointment\WP\Helpers as WPHelpers;
use Wappointment\Services\Admin;
use Wappointment\Services\AppointmentNew as Appointment;
use Wappointment\ClassConnect\Request;
use Wappointment\Services\DateTime;
class BookingController extends \Wappointment\Controllers\RestController
{
    public function save(\Wappointment\Validators\HttpRequest\Booking $booking)
    {
        if ($booking->hasErrors()) {
            return \Wappointment\WP\Helpers::restError('Review your fields', 500, $booking->getErrors());
        }
        $result = \Wappointment\Services\Client::book($booking);
        if (isset($result['appointment']['errors'])) {
            return \Wappointment\WP\Helpers::restError('Impossible to proceed with the booking', 500, $result['appointment']['errors']);
        }
        $result['result'] = true;
        $result['appointment'] = (new \Wappointment\ClassConnect\Collection($result['appointment']->toArraySpecial()))->except(['id', 'client_id']);
        return $result;
    }
    public function adminBook(\Wappointment\Validators\HttpRequest\BookingAdmin $booking)
    {
        if ($booking->hasErrors()) {
            return \Wappointment\WP\Helpers::restError('Review your fields', 500, $booking->getErrors());
        }
        $result = \Wappointment\Services\Admin::book($booking);
        if (isset($result['errors'])) {
            return \Wappointment\WP\Helpers::restError('Impossible to proceed with the booking', 500, $result['errors']);
        }
        return ['message' => 'Appointment recorded'];
    }
    public function reschedule(\Wappointment\ClassConnect\Request $request)
    {
        return \Wappointment\Services\AppointmentNew::reschedule($request->input('appointmentkey'), $request->input('time'));
    }
    public function convertDate(\Wappointment\ClassConnect\Request $request)
    {
        return ['converted' => $this->convert((int) $request->input('timestamp'), $request->input('timezone'))];
    }
    protected function convert($ts, $tz)
    {
        return \Wappointment\Services\DateTime::i18nDateTime($ts, $tz);
    }
}
