<?php

namespace Wappointment\Jobs;

use Wappointment\ClassConnect\Carbon;
use Wappointment\Services\AppointmentNew as AppointmentService;
use Wappointment\Services\Settings;
use Wappointment\Jobs\JobInterface;
use Wappointment\Services\Queue;
use Wappointment\Models\Job;
use Wappointment\Models\Order;
use Wappointment\Services\Payment;
class CleanPendingPaymentAppointment implements \Wappointment\Jobs\JobInterface
{
    public function handle()
    {
        if (\Wappointment\Services\Payment::isWooActive()) {
            //there is already an auto clean job for woocommerce
            return false;
        }
        // 1 - get orders that are pending for more than  the last X minutes
        $orders = \Wappointment\Models\Order::pending()->where('updated_at', '<', \Wappointment\ClassConnect\Carbon::now()->subSeconds(static::getDelayInSeconds())->format('Y-m-d H:i'))->get();
        // 3 - delete the connected appointments
        if (!empty($orders)) {
            foreach ($orders as $order) {
                foreach ($order->appointments as $appointment) {
                    if ($appointment->isPending()) {
                        \Wappointment\Services\AppointmentNew::cancel($appointment);
                    }
                }
                $order->setAutoCancelled();
            }
        }
        static::registerJob(true);
    }
    public static function registerJob($skipCheck = false)
    {
        if ($skipCheck || self::lastCheckExpired() && !self::hasJobInQueue('pending_clean')) {
            \Wappointment\Models\Job::where('queue', 'pending_clean')->delete();
            \Wappointment\Services\Queue::push('Wappointment\\Jobs\\CleanPendingPaymentAppointment', [], 'pending_clean', static::next());
        }
    }
    public static function lastCheckExpired()
    {
        return \Wappointment\Services\Settings::get('clean_last_check') === false || \Wappointment\Services\Settings::get('clean_last_check') < \time() - 5 * 60;
    }
    public static function hasJobInQueue($queue_key)
    {
        \Wappointment\Services\Settings::save('clean_last_check', \time());
        return \Wappointment\Models\Job::where('queue', $queue_key)->first();
    }
    public static function next()
    {
        return \time() + static::getDelayInSeconds();
    }
    public static function getDelayInSeconds()
    {
        return \Wappointment\Services\Settings::get('clean_pending_every') * 60;
    }
}
