<?php

namespace WappoVendor\libphonenumber\Leniency;

use WappoVendor\libphonenumber\PhoneNumber;
use WappoVendor\libphonenumber\PhoneNumberMatcher;
use WappoVendor\libphonenumber\PhoneNumberUtil;
class ExactGrouping extends \WappoVendor\libphonenumber\Leniency\AbstractLeniency
{
    protected static $level = 4;
    /**
     * Phone numbers accepted are PhoneNumberUtil::isValidNumber() valid and are grouped
     * in the same way that we would have formatted it, or as a single block. For example,
     * a US number written as "650 2530000" is not accepted at this leniency level, whereas
     * "650 253 0000" or "6502530000" are.
     * Numbers with more than one '/' symbol are also dropped at this level.
     *
     * Warning: This level might result in lower coverage especially for regions outside of country
     * code "+1". If you are not sure about which level to use, email the discussion group
     * libphonenumber-discuss@googlegroups.com.
     *
     * @param PhoneNumber $number
     * @param string $candidate
     * @param PhoneNumberUtil $util
     * @return bool
     */
    public static function verify(\WappoVendor\libphonenumber\PhoneNumber $number, $candidate, \WappoVendor\libphonenumber\PhoneNumberUtil $util)
    {
        if (!$util->isValidNumber($number) || !\WappoVendor\libphonenumber\PhoneNumberMatcher::containsOnlyValidXChars($number, $candidate, $util) || \WappoVendor\libphonenumber\PhoneNumberMatcher::containsMoreThanOneSlashInNationalNumber($number, $candidate) || !\WappoVendor\libphonenumber\PhoneNumberMatcher::isNationalPrefixPresentIfRequired($number, $util)) {
            return false;
        }
        return \WappoVendor\libphonenumber\PhoneNumberMatcher::checkNumberGroupingIsValid($number, $candidate, $util, function (\WappoVendor\libphonenumber\PhoneNumberUtil $util, \WappoVendor\libphonenumber\PhoneNumber $number, $normalizedCandidate, $expectedNumberGroups) {
            return \WappoVendor\libphonenumber\PhoneNumberMatcher::allNumberGroupsAreExactlyPresent($util, $number, $normalizedCandidate, $expectedNumberGroups);
        });
    }
}
