<?php

namespace WappoVendor\libphonenumber\Leniency;

use WappoVendor\libphonenumber\PhoneNumber;
use WappoVendor\libphonenumber\PhoneNumberMatcher;
use WappoVendor\libphonenumber\PhoneNumberUtil;
class StrictGrouping extends \WappoVendor\libphonenumber\Leniency\AbstractLeniency
{
    protected static $level = 3;
    /**
     * Phone numbers accepted are PhoneNumberUtil::isValidNumber() and are grouped
     * in a possible way for this locale. For example, a US number written as
     * "65 02 53 00 00" and "650253 0000" are not accepted at this leniency level, whereas
     * "650 253 0000", "650 2530000" or "6502530000" are.
     * Numbers with more than one '/' symbol in the national significant number are also dropped at
     * this level.
     *
     * Warning: This level might result in lower coverage especially for regions outside of country
     * code "+1". If you are not sure about which level to use, email the discussion group
     * libphonenumber-discuss@googlegroups.com.
     *
     * @param PhoneNumber $number
     * @param string $candidate
     * @param PhoneNumberUtil $util
     * @return bool
     */
    public static function verify(\WappoVendor\libphonenumber\PhoneNumber $number, $candidate, \WappoVendor\libphonenumber\PhoneNumberUtil $util)
    {
        if (!$util->isValidNumber($number) || !\WappoVendor\libphonenumber\PhoneNumberMatcher::containsOnlyValidXChars($number, $candidate, $util) || \WappoVendor\libphonenumber\PhoneNumberMatcher::containsMoreThanOneSlashInNationalNumber($number, $candidate) || !\WappoVendor\libphonenumber\PhoneNumberMatcher::isNationalPrefixPresentIfRequired($number, $util)) {
            return false;
        }
        return \WappoVendor\libphonenumber\PhoneNumberMatcher::checkNumberGroupingIsValid($number, $candidate, $util, function (\WappoVendor\libphonenumber\PhoneNumberUtil $util, \WappoVendor\libphonenumber\PhoneNumber $number, $normalizedCandidate, $expectedNumberGroups) {
            return \WappoVendor\libphonenumber\PhoneNumberMatcher::allNumberGroupsRemainGrouped($util, $number, $normalizedCandidate, $expectedNumberGroups);
        });
    }
}
